#!/bin/sh
#############################################################################
#
# Copyright (c) 2016 by SAS Institute Inc., Cary, NC 27513 USA
#  ---All Rights Reserved.
#
#############################################################################
#
# Description:
#     Shell script called by ending SAS Fraud Management batch job
#     to scan its log 
#
# Usage:
#     bat_scan_log2.sh -j <jobid> -d <target db> -r <jobrunid>
#                      -l <log filename>
#                      -p <bat_run_job.sh pid>
#                      -q <bat_run_job.sh -j parameter (job ID or alias)>
#
# Environment:
#     The following environment variables must be set for this script to
#     run:
#        SASFRAUD_HOME
#        IMPLEMENT_FOLDER
#
# Exit Status:
#     0  Success
#    >0  Failure
#
#############################################################################
#set -x

pid=$$
export tempdir="/tmp"
SASes="0"

#
# Source the user's profile ( cron doesn't necessarily do this )
# The assumption is that the above puts sas in path and defines the needed
# environment variables
#
#. ${HOME}/.profile

function errtrap {

   typeset es=$?
   
   echo "Submittal type is $submittal, line number=$1, es=$es" >> ${LOG_FILE} 

   if [ "$submittal" = "Scan lookup" ]
   then
      SASes=$es
      echo "WARNING line $1: $submittal exited with status $SASes." >> ${LOG_FILE}
   elif [ "$submittal" = "Scan run" ]
   then
      SASes=$es
      echo "WARNING line $1: $submittal exited with status $SASes." >> ${LOG_FILE}
   else
      echo "trap not from SAS exec es=$es."  >> ${LOG_FILE} 
      echo "WARNING line $1: trapped error $es." >> ${LOG_FILE} 
   fi
   }
   
trap 'errtrap $LINENO' ERR

pidp="";
jidp="";

#
# Parse the command line arguments
#

while getopts j:d:r:l:p:q: opt; do
   case $opt in
      j)    jobid=$OPTARG;;
      d)    db=$OPTARG;;
      r)    jobrun=$OPTARG;;
      l)    logloc=$OPTARG;;
      p)    pidp=$OPTARG;;
      q)    jidp=$OPTARG;;
   esac
done


if [ "$db" = "SOR" ] || [ "$db" = "sor" ]
then
   db="SOR";
   app_implroot="$SASFMCP_IMPLEMENT_HOME";
   app_autoexec="$app_implroot/common/etc/autoexec.sas";
   export TARGET_DB=sor;
elif [ "$db" = 'MEH' ] || [ "$db" = 'meh' ] 
then
   db="MEH";
   app_implroot="$SASOSE_IMPLEMENT_HOME";
   app_autoexec="$app_implroot/common/etc/meh_autoexec.sas";
   export TARGET_DB=meh;
else
   echo "$0: Error: unrecognized database acronym: '$db'" >> ${LOG_FILE}
   exit 1
fi

LOG_TIME=`date +%H%M%S`

######################################################
# Generate the file name for this shell script's log:
######################################################
LOG_DIR=$app_implroot/common/logs
LOG_DATE=`date +"%y_%m_%d"`
LOG_FTIME=`date +%H_%M`
LOG_FNAME=${LOG_DATE}_${LOG_FTIME}_job_2_${jidp}_shell_${pidp}.log
LOG_FILE=${LOG_DIR}/${LOG_FNAME}


#
# Log script start
#

echo "Job {$db:$jobid} submitted: ${LOG_DATE} ${LOG_TIME}" >> ${LOG_FILE}

if [ -z "${logloc}" ]; then
   echo "Job log for scanning not found." >> ${LOG_FILE}
   exit 3
fi


sysparm="jid=$jobid run=$jobrun log=$logloc";

#
# Invoke the SAS to run the requested job
#

submittal="Scan lookup";
scriptparm=$(sas_u8 -sysparm "pid=$pid pidp=$pidp jidp=$jidp job=2 db=$db sysparm=""$sysparm""" -echoauto -sysin "$SAS_ROOT/misc/fsmcm/source/bat_run_job2.sas" -LOGAPPLNAME "common" -LOG "job_2_${jidp}_lookup_${pidp}.log" -logconfigloc "$SAS_ROOT/misc/fsmcm/source/logconfig.trace.xml" -noterminal -autoexec "$app_autoexec" -unbuflog -nosyntaxcheck) 

wait

submittal="";

echo "returned scriptparm=$scriptparm" >> ${LOG_FILE}

if [ -x ${tempdir}/submit2_${pidp}.sh ]
then
   
   submittal="Scan run";
   ${tempdir}/submit2_${pidp}.sh
   submittal="";

   rm -f ${tempdir}/submit2_${pidp}.sh

else
   echo "Did not execute scan submit2_${pidp}.sh." >> ${LOG_FILE}
   exit 1
fi


#
# Log script complete
#
echo "Exiting with SAS rc=$SASes." >> ${LOG_FILE}
echo "Submit End: `date +"%y_%m_%d"` `date +%H%M%S`" >> ${LOG_FILE}
echo "" >> ${LOG_FILE}

exit $SASes

#
# End of File
#
###################################################################
