

call sysproc.set_routine_opts('ISOLATION UR') @

CREATE OR REPLACE PROCEDURE GET_ALERT ( IN i_alert_id BIGINT, 
                             IN i_user_name VARCHAR(60))
SPECIFIC GET_ALERT
DYNAMIC RESULT SETS 1
LANGUAGE SQL
-----------------------------------------------------------------------------
-- File:  GET_ALERT
--
--
-- Description:
--
--
-- Called By:
--     checkout_and_get_alert
--     get_next_alert
--     force_checkout
--     create_man_alert
--
-- Calls:
--     None
--
-- Notes:
--
--
-- Usage:
--
--
-- History:    10/17/2005               taken from FCMINT and stored in PRISM
--             2006/01/19   jicimi      removed account table from join
--             2006/06/02   jicimi      added outerjoin to fcm_rule table
--             2006/06/06               added logical_del_ind checks
--             2010/01/13   leushe      added the fcm_queue.disable_fco_ind column to the select
--             2010/02/13	leushe      added the inline selects to bring back the template ids based on templatetype code
--             2010/03/09   leushe      added 3 new queue fieldds to be returned queue_sys_block_days,queue_txn_block_days and queue_warning_msgs
--             2010/04/26   leushe      Changed to bring back the suspend user and number of suspenions for that grx_cust_num in the FCM_ALERT_SUSPEND table 
--             2010/10/22  leushe      Changed references to GRX_, GRA_ and GRC_ to XQO_,AQO_, and blank respectively
--             2011/02/24  leushe       Added rule_name to the query to be returned
--             2011/04/01   leushe      Changed the FCM_ALERT_SUSPEND to fix race condition and addition of ALERT_CHECKOUT_CNT column
--             2011/05/23   leushe      Removed base_rule_id, and rule_version_nbr being returned from the rule table, because its in the alert.
--             2013/07/07   leushe      Added root Multi_org_id on FCM_ALERT into the mix
--             2016/06/20   leushe      Added support for new columns LST_SERVICE_USER and EXPIRATION_TIMESTAMP 
--      	   2016/07/05    leushe      Added support for new columns LST_DISPOSITION_USER and LST_DISPOSITION_TIMESTAMP. Removed LST_SERVICE_USER 
--      	   2016/07/05    leushe      Added support for Global and Local Suspension
--      	   2016/08/24   leushe   ensured the property value retrieved is in lower case for comparison
--      	   2016/10/20   leushe   added TENANT_MULTI_ORG_NODE_KEY to be returned
------------------------------------------------------------------------



P1: BEGIN

    -- Declare cursor
    DECLARE cursor1 CURSOR WITH RETURN TO CLIENT FOR

	       
-- with suspension_details as (select property_value,alert_Type, LOCAL_SUSPENSION_IND                      
--from fcm_property, FCM_ALERT_TYPE
--where property_name ='analyst.alert.suspend.enabled')
 select aa.*, g.group_code, I_USER_NAME as user_name,fas2.suspend_user,fas2.ALERT_CHECKOUT_CNT as suspend_user_checkouts
         from ( select a.*,
                       s.strategy_id ,
                       s.group_id,
                       q.QUEUE_NAME ,
                       s.STRATEGY_NAME,
                       m.multi_org_name,
                       m.multi_org_node_key,
                       m.node_name,
                       pm.node_name  as TENANT_node_name,
                       pm.multi_org_name as TENANT_multi_org_name,
                       pm.multi_org_node_key as TENANT_MULTI_ORG_NODE_KEY,
                       case when (pm.parent_multi_org_id is null) then 'P' else 'T' end as TENANT_type,
                       r.alert_reason,
                       r.rule_name,
                       s.grid_template_id,
                       q.disable_fco_ind,
                       q.QUEUE_SYS_BLOCK_DAYS,
                       q.QUEUE_TXN_BLOCK_DAYS,
                       q.QUEUE_WARNING_MSG,
                       (Select tq.template_id 
                       			from fcm_template_queue tq, fcm_template t 
                       			where queue_id = q.queue_id                        			
                       			and tq.template_id = t.template_id 
                       			and default_template_ind = '1' 
                       			AND template_type_code = 'LTR') as DEFAULT_LETTER_ID ,
                       (Select tq.template_id 
                       			from fcm_template_queue tq, fcm_template t 
                       			where queue_id = q.queue_id 
                       			and tq.template_id = t.template_id 
                       			and default_template_ind = '1' 
                       			AND template_type_code = 'SMS') as DEFAULT_SMS_ID ,
                       (Select tq.template_id from fcm_template_queue tq, fcm_template t where  queue_id = q.queue_id and tq.template_id = t.template_id and    default_template_ind = '1' AND template_type_code = 'EML') as DEFAULT_EMAIL_ID ,
                       (Select ts.template_id from fcm_template_strategy ts, fcm_template t where ts.strategy_id = s.strategy_id and  ts.template_id = t.template_id  and default_template_ind = '1' AND template_type_code = 'CLS') as DEFAULT_CALLSCRIPT_ID ,
                       (Select ts.template_id from fcm_template_strategy ts, fcm_template t where ts.strategy_id = s.strategy_id and  ts.template_id = t.template_id  and     default_template_ind = '1' AND template_type_code = 'TPM') as DEFAULT_MEMO_ID                       
                from fcm_alert a left outer join fcm_rule r
                     on a.base_rule_id = r.base_rule_id and
                        a.rule_version_nbr = r.rule_version_nbr,
                     fcm_queue q, fcm_strategy s, fcm_multi_org m, fcm_multi_org pm
                where q.queue_id = a.queue_id and
                      q.strategy_id = s.strategy_id and 
                      m.multi_org_id = a.multi_org_id and
                      pm.multi_org_id = a.TENANT_MULTI_ORG_ID and
                      a.alert_id = I_ALERT_ID  and
                      a.logical_del_ind = '0' and
                      CHECK_MO_AT_ACCESS(I_USER_NAME,a.multi_org_id,a.alert_type) = 1
               ) as aa left join fcm_group g
                       on (aa.group_id = g.group_id)
               left join fcm_alert_suspend fas2
                     on (
						          		aa.contact_value = fas2.contact_value
                  						and aa.contact_type = fas2.contact_type
                  						and aa.TENANT_multi_org_id = fas2.TENANT_multi_org_id
                  						and fas2.suspend_user <> I_USER_NAME
                                        and suspension_level= (select case when local_Suspension_ind='1' then 'L' when lower(property_value)='true' then  'G' else '' end from fcm_property, FCM_ALERT_TYPE
                                                where property_name ='analyst.alert.suspend.enabled' and alert_Type=aa.alert_Type) 
                  						and aa.contact_Value is not null);


    -- Cursor left open for client application
    OPEN cursor1;
END P1@


