CALL SYSPROC.SET_ROUTINE_OPTS ('ISOLATION UR')@


CREATE OR REPLACE PROCEDURE RECLASSIFY_ALERT (
     IN I_SMH_ACCT_TYPE  CHARACTER(2),
     IN I_SMH_ACTIVITY_TYPE CHARACTER(2),     
     IN I_CMX_TRAN_ID BIGINT,
     IN I_ALERT_TYPE CHARACTER(1),
     IN I_ALERT_VALUE VARCHAR(100),
     IN I_PARENT_ALERT_TYPE CHARACTER(1),   --- If NO VALUE, there is no parent to set
     IN I_PARENT_VALUE VARCHAR(100),
     IN I_CONTACT_TYPE CHARACTER(1),        --- Will always have a value, but may not follow hierchary. The UI will validate the field is on the txn
     IN I_CONTACT_VALUE VARCHAR(100),
     IN I_MULTI_ORG_ID INTEGER,
     IN I_CMX_CLIENT_AMT  DECIMAL(19,5),
     IN I_CMX_MOD_AMT  DECIMAL(19,5),
     IN I_RQO_TRAN_DATE DATE,
     IN I_RQO_TRAN_TIME TIME,
     IN I_RQO_PROC_UTC_DATETIME TIMESTAMP,
     IN I_RRR_SCORE_1 INTEGER,
     IN I_RRR_SCORE_2 INTEGER,
     IN I_RRR_SCORE_3 INTEGER,
     IN I_RRR_SCORE_4 INTEGER,
     IN I_QUEUE_ID BIGINT,
     IN I_SRC_ALERT_ID BIGINT,
     IN I_SRC_ALERT_TYPE CHARACTER(1),
     IN I_SRC_ALERT_VERSION_NBR INTEGER,
     IN I_USER_NAME VARCHAR(60),
     IN I_NEED_TO_COMMIT    CHARACTER(1)    
    )
SPECIFIC RECLASSIFY_ALERT
DYNAMIC RESULT SETS 1
LANGUAGE SQL
-----------------------------------------------------------------------------
-- file:  RECLASSIFY_ALERT
--
--
-- description:
--    Creates a new Alert based on an existing alert. The UI is validating the USER_NAME has access to the ALERT_TYPE
--    being created. The proc needs to validate access to the MO of the txn.
--1.	Look up the field name for the alert type.
--2.	Look up the field name for the parent alert type.
--3.	Look up the field name for the contact type
--6.	Look up  GMT Offset based on contact value. 
--7.	If alert does not exist, insert alert
--8.	If alert exists and not closed, do nothing
--9.	If alert exists and is closed, reinit
--10.	Insert alert action
--11.	Return alert ID on success
--12.   Update txn with marking
--13.   Insert fmx_fraud_updt
--14.	Throw exception on failure.
--
-- calls:	   
--     get_alert
--     set_xact_fraud_flag
--     save_alert_action
--
-- notes:
--
--
-- usage:
--		Called from the web UI
--
--
-- history:
--       2012/06/08    leushe      Created for Version 3.3
--     2014/03/17    leushe      Added reference for FRAUD_SCORE_2-4 and remove FRAUD_REASON_CODES
--     2015/01/27    LEUSHE     set FINAL_DISPOSITION_TIMESTAMP=null when performing the update on the alert
--     2015/10/21    LEUSHE     Updated from SAS FM 4.3, NoRH. This proc will now only update/insert the Alert. The Transaction Update will be
--                              a seperate call from the UI to the Transaction Repository
--     2016/01/21    leushe     Fixed the insert when no alert found. 
--	   2016/01/25    leushe      changed signature of call to save_alert_action to include alert_status and action_type
--     2016/06/20    leushe      Added support for new columns LST_SERVICE_USER and EXPIRATION_TIMESTAMP 
--      2016/07/05    leushe      Added support for new columns LST_DISPOSITION_USER and LST_DISPOSITION_TIMESTAMP. Removed LST_SERVICE_USER
--     2017/02/07     leushe      On reinit, set LSTUPDT_NBR to 0 
------------------------------------------------------------------------

P1: BEGIN
    DECLARE SQLCODE INTEGER DEFAULT 0;
    DECLARE SQLSTATE CHAR(5) DEFAULT '00000';
	DECLARE L_SQLSTATE CHAR(5) DEFAULT '00000';
	DECLARE L_SQLCODE INTEGER;
    DECLARE L_TENANT_MULTI_ORG_ID INTEGER;
    DECLARE L_ROW_COUNT INTEGER DEFAULT -1;
    DECLARE L_CURRENT_UTC_TIMESTAMP TIMESTAMP;
    DECLARE L_ALERT_ACTION_STATUS VARCHAR(10) DEFAULT '';
    DECLARE L_ALERT_ID BIGINT DEFAULT -1;
    DECLARE L_ALERT_VERSION_NBR INTEGER DEFAULT 0;
    DECLARE L_ALERT_STATUS VARCHAR(10) DEFAULT 'ERROR' ;
    DECLARE L_BASESQL VARCHAR(2500);

    DECLARE ALERTCREATEFAILED CONDITION FOR SQLSTATE 'SASF2';
    DECLARE ALERTUPDATEFAILED CONDITION FOR SQLSTATE 'SASF3';
    
    DECLARE L_STMT STATEMENT;
    
    DECLARE c_stmt CURSOR for L_STMT;
    
    
    DECLARE EXIT HANDLER FOR alertCreateFailed
       RESIGNAL  SQLSTATE 'SASFM' SET MESSAGE_TEXT = 'Failed to Create new Alert';
    DECLARE EXIT HANDLER FOR alertUpdateFailed
       RESIGNAL  SQLSTATE 'SASFM' SET MESSAGE_TEXT = 'Failed to Re-Init Alert';

              
       --Set the current timestamp once
	   SET L_CURRENT_UTC_TIMESTAMP = CURRENT TIMESTAMP - CURRENT TIMEZONE;

 
  select tenant_multi_org_id into l_TENANT_multi_org_id from fcm_multi_org where  multi_org_id = i_multi_org_id;
          
    /*if we didn't set l_cmx_tran_id in the validation query above, bail*/
    set l_baseSql = 'select alert_id ,alert_version_nbr,alert_status,tenant_multi_org_id from FCM_ALERT  "
			+ " where  tenant_multi_org_id = ? and alert_type = ? and alert_value = ?';
   
      
    PREPARE l_stmt FROM l_baseSql;
		OPEN c_stmt using l_TENANT_multi_org_id,i_alert_type,i_alert_value;
		fetch from c_stmt into l_alert_id,l_alert_version_nbr,l_alert_status,l_TENANT_multi_org_id;
		SET l_sqlstate = SQLSTATE;
    close c_stmt;
    

	   

	    
	    
   		if (l_sqlstate = '00000') then
            if (l_alert_status = 'CLOSED') then
                set l_alert_version_nbr=l_alert_version_nbr+1;
				        set l_alert_action_status = 'INITCLOSED';
	 				UPDATE FCM_ALERT
           		 	SET LSTUPDT_NBR				    =0
           		 		,TENANT_MULTI_ORG_ID		=l_TENANT_multi_org_id
           		 		,SRC_ALERT_ID				=I_SRC_ALERT_ID 
						,SRC_ALERT_VERSION_NBR		=I_SRC_ALERT_VERSION_NBR 
						,SRC_ALERT_TYPE				=I_SRC_ALERT_TYPE
						,INIT_CMX_TRAN_ID			=I_CMX_TRAN_ID 
						,INIT_SMH_ACCT_TYPE			=I_SMH_ACCT_TYPE 
						,INIT_SMH_ACTIVITY_TYPE		=I_SMH_ACTIVITY_TYPE 
						,CONTACT_VALUE				=I_CONTACT_VALUE 
						,PARENT_ALERT_VALUE			=I_PARENT_VALUE 
						,CONTACT_TYPE				=I_CONTACT_TYPE 
						,PARENT_ALERT_TYPE			=I_PARENT_ALERT_TYPE 
						,RESULT_ID				    =NULL 
						,ALERT_STATUS				= 'ACTIVE'
						,ALERT_ORIGIN				= 'INBOUND'
						,BASE_RULE_ID			    = NULL
						,RULE_VERSION_NBR			= NULL
						,ALERT_SERVICE_CNT			= 0
						,MULTI_ORG_ID				= I_MULTI_ORG_ID
						,FINAL_DISPOSITION_TIMESTAMP	= NULL 
						,EXPIRATION_TIMESTAMP          = to_timestamp(to_char('9999-12-31-00'), 'YYYY-MM-DD-HH24')
            			,LST_TRANSACTION_TIMESTAMP		= (case when (i_rqo_tran_date is null or i_rqo_tran_Time is null) then i_rqo_proc_utc_datetime else timestamp(i_rqo_tran_date,i_rqo_tran_time) end)
						,INIT_TRANSACTION_TIMESTAMP		= (case when (i_rqo_tran_date is null or i_rqo_tran_Time is null) then i_rqo_proc_utc_datetime else timestamp(i_rqo_tran_date,i_rqo_tran_time) end)
						,LST_TRANSACTION_AMT			= I_CMX_CLIENT_AMT
						,INIT_TRANSACTION_AMT			= I_CMX_CLIENT_AMT
						,LST_TRANSACTION_MOD_AMT			= I_CMX_MOD_AMT
						,INIT_TRANSACTION_MOD_AMT			= I_CMX_MOD_AMT
						,FRAUD_SCORE_1				        = I_RRR_SCORE_1
						,HWM_FRAUD_SCORE_1				    = I_RRR_SCORE_1
						,INIT_FRAUD_SCORE_1				    = I_RRR_SCORE_1
						,FRAUD_SCORE_2				        = I_RRR_SCORE_2
						,HWM_FRAUD_SCORE_2				    = I_RRR_SCORE_2
						,INIT_FRAUD_SCORE_2				    = I_RRR_SCORE_2
						,FRAUD_SCORE_3				        = I_RRR_SCORE_3
						,HWM_FRAUD_SCORE_3				    = I_RRR_SCORE_3
						,INIT_FRAUD_SCORE_3				    = I_RRR_SCORE_3
						,FRAUD_SCORE_4				        = I_RRR_SCORE_4
						,HWM_FRAUD_SCORE_4				    = I_RRR_SCORE_4
						,INIT_FRAUD_SCORE_4				    = I_RRR_SCORE_4
						,PAUSE_TIMESTAMP				    = NULL
						,PAUSE_ELAPSED_TIME				    = NULL
						,QUEUE_ID				            = I_QUEUE_ID
						,ALERT_VERSION_NBR				    = l_alert_version_nbr
						,CHECKOUT_TIMESTAMP				    = NULL
						,CREATE_USER				        = I_USER_NAME
						,CREATE_TIMESTAMP				    = L_CURRENT_UTC_TIMESTAMP
						,LST_DISPOSITION_USER 				= null
						,LST_DISPOSITION_TIMESTAMP          =null
						,LSTUPDT_USER				        = I_USER_NAME
						,LSTUPDT_TIMESTAMP            	    = L_CURRENT_UTC_TIMESTAMP 
					WHERE
						ALERT_ID = l_ALERT_ID;

			        set l_sqlcode=SQLCODE;
           		SET l_sqlstate = SQLSTATE;
   		   		if (l_sqlstate != '00000') then
            		SIGNAL alertUpdateFailed;
   		   		END IF;

        		
       					
 													
            end if;
   		
   		else
   		    set l_alert_version_nbr=0;
   		    set l_alert_action_status = 'ALERTINIT';
   			
   		    select alert_id into l_alert_id from FINAL TABLE(
   			
	   		    INSERT into fcm_alert (TENANT_MULTI_ORG_ID,ALERT_VERSION_NBR, ALERT_VALUE, ALERT_TYPE, MULTI_ORG_ID, CONTACT_VALUE, PARENT_ALERT_VALUE
	                , QUEUE_ID, ALERT_STATUS, GMT_OFFSET, FRAUD_SCORE_1, HWM_FRAUD_SCORE_1, INIT_FRAUD_SCORE_1
	                , FRAUD_SCORE_2, HWM_FRAUD_SCORE_2, INIT_FRAUD_SCORE_2
	                , FRAUD_SCORE_3, HWM_FRAUD_SCORE_3, INIT_FRAUD_SCORE_3
	                , FRAUD_SCORE_4, HWM_FRAUD_SCORE_4, INIT_FRAUD_SCORE_4
	            	, ALERT_SERVICE_CNT, ALERT_ORIGIN, INIT_TRANSACTION_TIMESTAMP, INIT_TRANSACTION_AMT, LST_TRANSACTION_AMT, LST_TRANSACTION_TIMESTAMP
	            	, INIT_TRANSACTION_MOD_AMT, LST_TRANSACTION_MOD_AMT,NEXT_SERVICE_TIMESTAMP, QUEUE_ENTRY_TIMESTAMP, CREATE_USER, CREATE_TIMESTAMP
	            	, LSTUPDT_USER, LSTUPDT_TIMESTAMP,LSTUPDT_NBR, LOGICAL_DEL_IND, INIT_CMX_TRAN_ID, INIT_SMH_ACCT_TYPE, INIT_SMH_ACTIVITY_TYPE
	            	, CONTACT_TYPE,PARENT_ALERT_TYPE,SRC_ALERT_ID,SRC_ALERT_VERSION_NBR,SRC_ALERT_TYPE) 
	            values (l_TENANT_multi_org_id,l_alert_version_nbr,I_ALERT_VALUE,i_alert_type,i_multi_org_id,i_contact_value,I_PARENT_VALUE
	            		,i_queue_id,'ACTIVE',0,i_rrr_score_1,i_rrr_score_1,i_rrr_score_1
	            		,i_rrr_score_2,i_rrr_score_2,i_rrr_score_2
	            		,i_rrr_score_3,i_rrr_score_3,i_rrr_score_3
	            		,i_rrr_score_4,i_rrr_score_4,i_rrr_score_4
	            		,0,'INBOUND',(case when (i_rqo_tran_date is null or i_rqo_tran_Time is null) then i_rqo_proc_utc_datetime else timestamp(i_rqo_tran_date,i_rqo_tran_time) end)
	            		,i_cmx_client_amt,i_cmx_client_amt, (case when (i_rqo_tran_date is null or i_rqo_tran_Time is null) then i_rqo_proc_utc_datetime else timestamp(i_rqo_tran_date,i_rqo_tran_time) end)
	            		,i_cmx_mod_amt,i_cmx_mod_amt,L_CURRENT_UTC_TIMESTAMP,L_CURRENT_UTC_TIMESTAMP,I_USER_NAME,L_CURRENT_UTC_TIMESTAMP
	            		,I_USER_NAME,L_CURRENT_UTC_TIMESTAMP,0,'0',i_cmx_tran_id,I_SMH_ACCT_TYPE,I_SMH_ACTIVITY_TYPE
	            		,I_contact_type,i_parent_alert_type,I_SRC_ALERT_ID,I_SRC_ALERT_VERSION_NBR,I_SRC_ALERT_TYPE)
            		);
		    
		   		   
           set l_sqlcode=SQLCODE;
           SET l_sqlstate = SQLSTATE;
   		   if (l_sqlstate != '00000') then
            	SIGNAL alertCreateFailed;
   		   END IF;
 
        
    end if;
	
    IF (length(L_ALERT_ACTION_STATUS) > 0) THEN
    		 call save_alert_action (l_alert_id,L_ALERT_ACTION_STATUS,'OUT','N');
    END IF;
	

	
     IF (I_NEED_TO_COMMIT = 'Y') THEN
     	COMMIT;
     END IF;
     
     CALL GET_ALERT(L_ALERT_ID, I_USER_NAME);
     -- eventually will need to signal an exception; now just return an empty set

  
END P1 @

