CREATE OR REPLACE FUNCTION decode_base64_signature (VARIABLE_DATA_00 varchar2               
        , VARIABLE_DATA_01 varchar2               
        , VARIABLE_DATA_02  varchar2              
        , VARIABLE_DATA_03  varchar2              
        , VARIABLE_DATA_04 varchar2               
        , VARIABLE_DATA_05 varchar2               
        , VARIABLE_DATA_06 varchar2               
        , VARIABLE_DATA_07 varchar2               
)
RETURN BLOB IS
-----------------------------------------------------------------------------
-- File:  GET_VXX
--
--
-- Description:  For the given CMX_TRAN_ID,RQO_PROC_UTC_DATETIME, the function executes a query against the Table Name passed in
--               and returns the build_num and BLOB of the VXX data for the given UVR_IDs passed in.
--               The function then pivots the data into one row
--
--
-- Called By:    Web App
--
--
-- Calls:
--
-- Notes:
--
--
-- Usage:
--		SELECT vxx_uvr_id,vxx_build_num
--             ,decode_base64_signature (VARIABLE_DATA_00,VARIABLE_DATA_01,VARIABLE_DATA_02,VARIABLE_DATA_03,VARIABLE_DATA_04,VARIABLE_DATA_05,VARIABLE_DATA_06,VARIABLE_DATA_07)
--      from FCM_CCCA_VXX where cmx_tran_id = :1 and rqo_proc_utc_datetime = :2 and vxx_uvr_id in ('01','02') order by vxx_uvr_id
--
-- History:    2017/1/12:  Creation date
--
------------------------------------------------------------------------
	my_clob CLOB;
	step  VARCHAR2(2);
    v_blob blob;
    v_result blob;
    v_offset integer;
    v_buffer_size binary_integer := 1500;
    v_buffer_varchar varchar2(1500);
    v_buffer_raw raw(1500);


 BEGIN
    -- Create Temporary LOBs
      dbms_lob.createtemporary(my_clob, TRUE);
      dbms_lob.createtemporary(v_blob, true);

      dbms_lob.append(my_clob,coalesce(VARIABLE_DATA_00,'')||coalesce(VARIABLE_DATA_01,'')||coalesce(VARIABLE_DATA_02,'')||coalesce(VARIABLE_DATA_03,'')||coalesce(VARIABLE_DATA_04,'')||coalesce(VARIABLE_DATA_05,'')||coalesce(VARIABLE_DATA_06,'')
                ||coalesce(VARIABLE_DATA_07,'')
                );

      
        v_offset := 1;
        for i in 1 .. ceil(dbms_lob.getlength(my_clob) / v_buffer_size) loop
            dbms_lob.read(my_clob, v_buffer_size, v_offset, v_buffer_varchar);
            v_buffer_raw := utl_raw.cast_to_raw(v_buffer_varchar);
            v_buffer_raw := utl_encode.base64_decode(v_buffer_raw);
            dbms_lob.writeappend(v_blob, utl_raw.length(v_buffer_raw), v_buffer_raw);
            v_offset := v_offset + v_buffer_size;
        end loop;
    v_result := v_blob;

    -- Free Temporary LOBs   
    dbms_lob.freetemporary(my_clob);
    dbms_lob.freetemporary(v_blob);
    return v_result;
    
   EXCEPTION
     WHEN NO_DATA_FOUND THEN
       NULL;
     WHEN OTHERS THEN
        return  null ;
    --    RAISE;
END decode_base64_signature;
/
