DROP FUNCTION GET_VXX;
DROP FUNCTION decode_base64_VXX;
DROP TYPE vxx_row_table;
DROP TYPE Vxx_row;

CREATE OR REPLACE TYPE signature_row AS OBJECT
       (CMX_TRAN_ID         numeric(19),
        RQO_PROC_UTC_DATETIME timestamp,
        encoding varchar2(1),        
		signature00_build_num number(10),
		signature01_build_num number(10),
		signature02_build_num number(10),
		signature03_build_num number(10),
		signature04_build_num number(10),
		signature05_build_num number(10),
		signature06_build_num number(10),
		signature07_build_num number(10),
		signature08_build_num number(10),
		signature09_build_num number(10),
		signature10_build_num number(10),
		signature11_build_num number(10),
		signature12_build_num number(10),
		signature13_build_num number(10),
		signature14_build_num number(10),
		signature15_build_num number(10),
		signature16_build_num number(10),
		signature17_build_num number(10),
		signature18_build_num number(10),
		signature19_build_num number(10),
		signature20_build_num number(10),
		signature21_build_num number(10),
		signature22_build_num number(10),
		signature23_build_num number(10),
		signature24_build_num number(10),
		signature25_build_num number(10),
		signature26_build_num number(10),
		signature27_build_num number(10),
		signature28_build_num number(10),
		signature29_build_num number(10),
		signature30_build_num number(10),
		signature31_build_num number(10),
		signature32_build_num number(10),
		signature33_build_num number(10),
		signature00 blob,
		signature01 blob,
		signature02 blob,
		signature03 blob,
		signature04 blob,
		signature05 blob,
		signature06 blob,
		signature07 blob,
		signature08 blob,
		signature09 blob,
		signature10 blob,
		signature11 blob,
		signature12 blob,
		signature13 blob,
		signature14 blob,
		signature15 blob,
		signature16 blob,
		signature17 blob,
		signature18 blob,
		signature19 blob,
		signature20 blob,
		signature21 blob,
		signature22 blob,
		signature23 blob,
		signature24 blob,
		signature25 blob,
		signature26 blob,
		signature27 blob,
		signature28 blob,
		signature29 blob,
		signature30 blob,
		signature31 blob,
		signature32 blob,
		signature33 blob,
		stmt_text varchar2(1000));
/
CREATE TYPE signature_row_table AS TABLE OF signature_row;
/



CREATE OR REPLACE FUNCTION GET_SIGNATURE (i_tran_id numeric,i_proc_utc_datetime timestamp,i_table_name varchar2,i_segment_id varchar2) 
         RETURN signature_row_table pipelined IS
-----------------------------------------------------------------------------
-- File:  GET_SIGNATURE
--
--
-- Description:  For the given CMX_TRAN_ID,RQO_PROC_UTC_DATETIME, the function executes a query against the Table Name passed in
--               and returns the build_num and BLOB of the signature data for the given UVR_IDs passed in.
--               The function then pivots the data into one row, of type signature_row
--
--
-- Called By:    webapp Estimation process.
--
--
-- Calls:
--				decode_base64_signature
--
-- Notes:
--
--
-- Usage:
--		with tran as (select distinct                                           /*---- USED DISTINCT HERE SO WE DONT GET REPEATING CMX_TRAN_IDS ----*/
--		  fcm_csca.CMX_TRAN_ID ,
--		  fcm_csca.rqo_proc_utc_datetime,
--		  RQO_SEG_ID_VERSION,
--		  CMX_MOD_AMT 
--		  from FCM_CSCA inner join fcm_csca_vxx                                   /*----- I DID THIS INNER JOIN SO THAT WE DONT LOOK FOR vxx RECORDS NOT HERE */
--		on fcm_csca.cmx_tran_id=fcm_csca_vxx.cmx_Tran_id
--		      and fcm_csca.rqo_proc_utc_datetime = fcm_csca_vxx.rqo_proc_utc_datetime
--		)
--		  select cast(TRAN.CMX_TRAN_ID as varchar2(20)) ,
--		  TRAN.RQO_SEG_ID_VERSION,
--		  TRAN.CMX_TRAN_ID,
--		  TRAN.CMX_MOD_AMT ,
--        VXX.ENCODING,
--		  VXX.signature00_build_num,
--		  VXX.signature00,
--		  VXX.signature01,
--		  VXX.signature02,
--		  VXX.signature03,
--		  VXX.signature04,
--		  VXX.signature05,
--		  VXX.signature06,
--		  VXX.signature07
--		  FROM
--		     TRAN
--		    INNER JOIN  ( TABLE(GET_SIGNATURE(TRAN.CMX_TRAN_ID,TRAN.RQO_PROC_UTC_DATETIME,'FCM_CSCA_VXX','00,01')) )  VXX     /*--- GET_SIGNATURE FUNCTION HAS 3 ARGUMENTS. NOTE, IT ONLY SUPPORTS 8 UV SEGMENTS,*/
--		      ON TRAN.CMX_TRAN_ID=VXX.CMX_TRAN_ID;    
--
--
-- History:    2015/11/16:  Creation date
--				2017/02/28  Added Encoding from the signature table as an output column
------------------------------------------------------------------------
TYPE         ref0 IS REF CURSOR;
cur0         ref0;
out_rec      signature_row := signature_row(NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL
                  ,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL
                  ,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL
                  ,NULL,NULL,NULL,NULL,NULL,NULL,
                  NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL
                  ,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL
                  ,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL
                  ,NULL,NULL,NULL,NULL,null);

segment_id varchar2(2);
signature_blob blob;
build_num numeric(10);
encoding varchar2(1);
table_suffix varchar2(3);
segment_id_col_name varchar2(10);
BEGIN

table_suffix:=substr(i_table_name,length(i_table_name)-2);
   IF (UPPER (table_suffix) = 'VXX')
   THEN
      segment_id_col_name := '_uvr_id';
   ELSE
      segment_id_col_name := '_id';
   END IF;
   
OPEN cur0 FOR 
  'SELECT '||table_suffix||segment_id_col_name||','||table_suffix||'_build_num,encoding,decode_base64_signature (VARIABLE_DATA_00,VARIABLE_DATA_01,VARIABLE_DATA_02,VARIABLE_DATA_03,VARIABLE_DATA_04,VARIABLE_DATA_05,VARIABLE_DATA_06,VARIABLE_DATA_07)'
  ||' from '||i_table_name||' where cmx_tran_id = :1 and rqo_proc_utc_datetime = :2 and '||table_suffix||segment_id_col_name||' in ('||i_segment_id||') order by '||table_suffix||segment_id_col_name||''
USING i_tran_id,i_proc_utc_datetime;
  LOOP
    FETCH cur0 INTO segment_id, build_num, encoding,  signature_blob;
    EXIT WHEN cur0%NOTFOUND;
    case 
    WHEN (segment_id='00') then         out_rec.signature00:=signature_blob;        out_rec.signature00_build_num:=build_num;  
    WHEN (segment_id='01') then         out_rec.signature01:=signature_blob;        out_rec.signature01_build_num:=build_num;
    WHEN (segment_id='02') then         out_rec.signature02:=signature_blob;        out_rec.signature02_build_num:=build_num;
    WHEN (segment_id='03') then         out_rec.signature03:=signature_blob;        out_rec.signature03_build_num:=build_num;
    WHEN (segment_id='04') then         out_rec.signature04:=signature_blob;        out_rec.signature04_build_num:=build_num;
    WHEN (segment_id='05') then         out_rec.signature05:=signature_blob;        out_rec.signature05_build_num:=build_num;
    WHEN (segment_id='06') then         out_rec.signature06:=signature_blob;        out_rec.signature06_build_num:=build_num;
    WHEN (segment_id='07') then         out_rec.signature07:=signature_blob;        out_rec.signature07_build_num:=build_num;
    WHEN (segment_id='08') then         out_rec.signature08:=signature_blob;        out_rec.signature08_build_num:=build_num;  
    WHEN (segment_id='09') then         out_rec.signature09:=signature_blob;        out_rec.signature09_build_num:=build_num;
    WHEN (segment_id='10') then         out_rec.signature10:=signature_blob;        out_rec.signature10_build_num:=build_num;
    WHEN (segment_id='11') then         out_rec.signature11:=signature_blob;        out_rec.signature11_build_num:=build_num;
    WHEN (segment_id='12') then         out_rec.signature12:=signature_blob;        out_rec.signature12_build_num:=build_num;
    WHEN (segment_id='13') then         out_rec.signature13:=signature_blob;        out_rec.signature13_build_num:=build_num;
    WHEN (segment_id='14') then         out_rec.signature14:=signature_blob;        out_rec.signature14_build_num:=build_num;
    WHEN (segment_id='15') then         out_rec.signature15:=signature_blob;        out_rec.signature15_build_num:=build_num;
    WHEN (segment_id='16') then         out_rec.signature16:=signature_blob;        out_rec.signature16_build_num:=build_num;  
    WHEN (segment_id='17') then         out_rec.signature17:=signature_blob;        out_rec.signature17_build_num:=build_num;
    WHEN (segment_id='18') then         out_rec.signature18:=signature_blob;        out_rec.signature18_build_num:=build_num;
    WHEN (segment_id='19') then         out_rec.signature19:=signature_blob;        out_rec.signature19_build_num:=build_num;
    WHEN (segment_id='20') then         out_rec.signature20:=signature_blob;        out_rec.signature20_build_num:=build_num;
    WHEN (segment_id='21') then         out_rec.signature21:=signature_blob;        out_rec.signature21_build_num:=build_num;
    WHEN (segment_id='22') then         out_rec.signature22:=signature_blob;        out_rec.signature22_build_num:=build_num;
    WHEN (segment_id='23') then         out_rec.signature23:=signature_blob;        out_rec.signature23_build_num:=build_num;
    WHEN (segment_id='24') then         out_rec.signature24:=signature_blob;        out_rec.signature24_build_num:=build_num;  
    WHEN (segment_id='25') then         out_rec.signature25:=signature_blob;        out_rec.signature25_build_num:=build_num;
    WHEN (segment_id='26') then         out_rec.signature26:=signature_blob;        out_rec.signature26_build_num:=build_num;
    WHEN (segment_id='27') then         out_rec.signature27:=signature_blob;        out_rec.signature27_build_num:=build_num;
    WHEN (segment_id='28') then         out_rec.signature28:=signature_blob;        out_rec.signature28_build_num:=build_num;
    WHEN (segment_id='29') then         out_rec.signature29:=signature_blob;        out_rec.signature29_build_num:=build_num;
    WHEN (segment_id='30') then         out_rec.signature30:=signature_blob;        out_rec.signature30_build_num:=build_num;
    WHEN (segment_id='31') then         out_rec.signature31:=signature_blob;        out_rec.signature31_build_num:=build_num;
    WHEN (segment_id='32') then         out_rec.signature32:=signature_blob;        out_rec.signature32_build_num:=build_num;
    WHEN (segment_id='33') then         out_rec.signature33:=signature_blob;        out_rec.signature33_build_num:=build_num;        
    end case;
    out_rec.encoding:=encoding;
    
    EXIT WHEN cur0%NOTFOUND;
       
    
  END LOOP;
  
  CLOSE cur0;
  out_rec.cmx_tran_id:=i_tran_id;
  out_rec.stmt_text:='SELECT '||table_suffix||segment_id_col_name||',decode_base64_signature (VARIABLE_DATA_00,VARIABLE_DATA_01,VARIABLE_DATA_02,VARIABLE_DATA_03,VARIABLE_DATA_04,VARIABLE_DATA_05,VARIABLE_DATA_06,VARIABLE_DATA_07)'
  ||' from '||i_table_name||' where cmx_tran_id = :1 and rqo_proc_utc_datetime = :2 and '||table_suffix||segment_id_col_name||' in ('||i_segment_id||') order by '||table_suffix||segment_id_col_name||'';
  out_rec.rqo_proc_utc_datetime:=i_proc_utc_datetime;
    PIPE ROW(out_rec);
RETURN;
END GET_SIGNATURE;
/