%macro fix_numeric;

**** ensure that all macro variables and temporary data is deleted at bottom of macro *****;
**** macro variables coming from proc sql in this scenario are global *****;


call execute("%let numover10=;");
call execute("%let totpercentover10=;");
call execute("%let totalbuckets=;");
call execute("%let bucket_percent=;");

*** collapse Numeric Data ****;

call execute("proc freq data="||strip(Plot_data)||";");
call execute(" tables "||strip(report_variable)||"/missing out=temp_dist NOPRINT;");
call execute(" weight percent;");
call execute(" run;");

***** algorithm for calculating % in collapsed buckets ******;

call execute(" proc sql noprint;");
call execute(" select count(*), sum(percent)");
call execute(" into :numover10, :totpercentover10");
call execute(" from temp_dist");
call execute(" where percent>10;");

call execute(" select count(*)");
call execute(" into :totalbuckets");
call execute(" from temp_dist;");
call execute("quit;");

**** the following adds a 'newgroup' - variables with less than 5% in each cell are collapsed *****;

call execute(" data temp_dist2;");
call execute("  set temp_dist nobs=numobs;");
call execute("  drop startflag cum_temp_percent numover10 totpercentover10 totalbuckets;");
call execute("  retain startflag 0;");
call execute("length numover10 totpercentover10 totalbuckets 8;");


call execute('numover10=SYMGET("numover10");');
call execute('totpercentover10=SYMGET("totpercentover10");');
call execute('totalbuckets=SYMGET("totalbuckets");');
call execute("if totalbuckets>numover10 then bucket_percent=totpercentover10/(totalbuckets-numover10);");
call execute("if numover10<1 then bucket_percent=10;");
/**call execute("put _all_;"); **--debugging tool--**/

call execute("if totalbuckets>10 then do;");
    call execute(" if percent> bucket_percent then newgroup+1;");
	call execute(" else if cum_temp_percent< bucket_percent and startflag=0 then do;");
	call execute("   newgroup+1; ");
	call execute("   startflag=1;");
	call execute("   cum_temp_percent+percent;");
	call execute(" end;");
	call execute(" else if cum_temp_percent< bucket_percent and startflag=1 then cum_temp_percent+percent;");
	call execute(" if cum_temp_percent> bucket_percent then do;");
	call execute("    output;");
	call execute("    startflag=0;");
	call execute("    cum_temp_percent=0;");
	call execute(" end;");
	call execute(" else output;");
call execute("end;");
call execute("else if percent>0 then do; newgroup=_n_; output; end;");
call execute(" run;");


**** determines the max and min values in the new group;

call execute("proc sql noprint;");
call execute("create table group_hash(drop=min max) as");
call execute("select newgroup, min("||strip(report_variable)||") as min, max("||strip(report_variable)||") as max, strip(put(calculated min,"||strip(report_variable_format)||")) as min_char,");
call execute("             strip(put(calculated max,"||strip(report_variable_format)||")) as max_char ");                          
call execute("from temp_dist2");
call execute("group by newgroup;");

**** creates a label based on the max and min variables in the group;
***** adds the newgroup and its accompanying label back to the plot data *******;

call execute("data group_hash2;");
call execute(" set group_hash;");
call execute(" length label $50;");
call execute(" if min_char=max_char then label=strip(min_char);");
call execute(" else label=strip(cats(scan(min_char,1,'-'),'-',scan(max_char,-1,'-')));");
call execute(" run;");

call execute("data "||strip(plot_data)||";");
call execute("set "||strip(plot_data)||";");

call execute(" length newgroup 8; call missing(newgroup);");

call execute(" if _n_=1 then do;");
call execute("   declare hash addgrp(dataset: 'temp_dist2');");
call execute("    addgrp.defineKey('"||strip(report_variable)||"');");
call execute("    addgrp.defineData('newgroup');");
call execute("	addgrp.definedone();");
call execute(" end;");

call execute(" rc1=addgrp.find(); ");
call execute(" length label $50.; call missing(label);");

call execute(" if _n_=1 then do;");
call execute("  declare hash mtlabel(dataset: 'group_hash2');");
call execute("   mtlabel.defineKey('newgroup');");
call execute("   mtlabel.defineData('label');");
call execute("   mtlabel.definedone();");
call execute(" end;");

call execute(" rc2=mtlabel.find();");
call execute(" drop "||strip(report_variable)||";");
call execute("sort_var="||strip(report_variable)||";");
call execute(" run;");

******* Sort and Index HERE *****;

call execute("proc sort data="||strip(plot_data)||";");
call execute("by sort_var;");
call execute("run;");

if upcase(run)='DRILL' then do;
     call execute("data "||strip(plot_data)||"(index=(drill_var_"||strip(drill_number)||" group));");
	 call execute(" set "||strip(plot_data)||";");
end;
else do; 
	call execute("data "||strip(plot_data)||"(index=(group));");
    call execute(" set "||strip(plot_data)||";");
end;
    call execute("run;");


call execute(" proc datasets library=work nolist nodetails;");;
call execute("  modify "||strip(plot_data)||";");
call execute("  rename label="||strip(report_variable)||";");
call execute("  run;quit;");


call execute("proc sql noprint;");
call execute("create table format as");
call execute(" select distinct sort_var as start, "||strip(report_variable)||" as label, UPCASE('TMP_'||STRIP('"||strip(substr(report_variable,1,27))||"_')) as fmtname, 'N' as type");
 call execute(" from "||strip(plot_data)||";");

call execute("proc format cntlin=format out=work;");
call execute("run;");



%mend fix_numeric;
