/*
 * extract development for analytic reporting of performance evaluation
 *
 * Author: Song, Qiang <qiang.song@sas.com> 2015
 *
 */

%macro rpt_extract_prod_data_quarterly(
    frh_table =,             /* the output dataset will be default to work.&frh_table; */
    out =,             /* the output dataset will be default to work.&frh_table; */
    wherecond = 1,        /* conditions to subsetting input dataset */
    rh_sampling_rate = 1,          /* non-fraud case sampling rate */
    rh_sampling_rate_fraud = 1,          /* fraud case sampling rate */
    keeplist = ,
    rhkeeplist =
    );


    %local sampling_dig wts_var;

    %let sampling_dig = %sysevalf(1000 * &rh_sampling_rate, ceil);
    %let wts_var = %sysevalf(1000 / &sampling_dig, ceil);

    %let sampling_dig_fraud = %sysevalf(1000 * &rh_sampling_rate_fraud, ceil);
    %let wts_var_fraud = %sysevalf(1000 / &sampling_dig_fraud, ceil);

    /* connect to analytic datasets */
    %if %nrbquote(&ANALYTIC_EXTRACT_DATASTORE) ne %then 
    %do;
        libname dblib "&ANALYTIC_EXTRACT_DATASTORE";        
    %end;      
    /* connect to RH database and extract original data */
    %else %if %nrbquote(&fake_rh_dir) ne %then 
    %do;
        %fake_connect_to_db(lib = dblib, dir = &fake_rh_dir);
    %end;
    %else %do;
        %connect_to_db(lib = dblib);
    %end;
    
    data _tmp_trans_(keep = &keeplist);
        set dblib.&frh_table(keep = &rhkeeplist %fsmetl_getext);
        where
            (&wherecond) and
            ((wax_score_1_tag in ('2', '1', '3') and hqo_sas_card_rand_dig < &sampling_dig_fraud)
             or (not(wax_score_1_tag in ('2', '1', '3')) and hqo_sas_card_rand_dig < &sampling_dig))
            ;
 
        /* analytic datasets are already converted */
         %if %nrbquote(&ANALYTIC_EXTRACT_DATASTORE) eq %then 
         %do;
           %db_specific_data_conversion; 
         %end;  
        
        if missing(wax_score_1_tag) then wax_score_1_tag = '0';

        if wax_score_1_tag eq '0' then
            wts_var = &wts_var;
        else
            wts_var = &wts_var_fraud;


        /* use RHX fields to populate AFD fields to fit the evaluation macro API */
        length AFD_FRD_TAG AFD_FRD_TAG_ORIG AFD_MATCH_FRD_TXN $1
               AFD_FRD_TYPE $2
               AFD_BLOCK_DATE AFD_BLOCK_TIME AFD_DOFF AFD_DOFF_TIME 8;

        call missing(OF AFD_:);

        AFD_FRD_TAG = wax_score_1_tag;
        AFD_MATCH_FRD_TXN = ifc(missing(wax_score_1_sas_match_ind) or wax_score_1_sas_match_ind eq '00', 'N', 'M');
    run;
    libname dblib clear;

************************ put tag adjustment fix here ************************ ;

    proc sql;
     create table non_zero_tagged_cards as
     select hqo_card_num,
     (sum(afd_frd_tag in('1','2','3'))>=1) as card_flag
     from _tmp_trans_
     group by hqo_card_num
     having calculated card_flag=1;

   data _tmp_trans_;       
       *** add hash to identify cards with tags ***;
	if _n_=1 then do;
	if 0 then set work.non_zero_tagged_cards(keep=hqo_card_num);  call missing(hqo_card_num);
		declare hash x(dataset: 'work.non_zero_tagged_cards(keep=hqo_card_num)');
		  x.definekey('hqo_card_num');
		x.definedone();
	end;    
        set _tmp_trans_;

        rc=x.find(); drop rc;
        *** for cards that match the non_zero_tagged_cards hash - reassign and missing tags or zero tags to 3 **;
        if rc=0 then do;
            if missing(afd_frd_tag) or afd_frd_tag='0' then afd_frd_tag='3';
        end;
   run;

***************************************************************************;  


    /* for marked cards, estimate first fraud and block datetime */
    data trans_marked / view = trans_marked;
        set _tmp_trans_(where = (afd_frd_tag ne '0'));
    run;

    %get_frd_window_dtm(in = trans_marked, out = _tmp_marked_cards);

    data &out(keep = &keeplist);
       
	    if 0 then set _tmp_marked_cards;
	    call missing(of _all_);
            
		if _n_ = 1 then do;
            declare hash marked_cards(dataset:'_tmp_marked_cards', hashexp: 16);
            marked_cards.defineKey('hqo_card_num');
            marked_cards.defineData('dtm_ff', 'dtm_blk');
            marked_cards.defineDone();
        end;

        set _tmp_trans_;

        if afd_frd_tag ne '0' then do;
            rc = marked_cards.find();
            if rc = 0 then do;
                afd_doff = datepart(dtm_ff);
                afd_doff_time = timepart(dtm_ff);
                afd_block_date = datepart(dtm_blk);
                afd_block_time = timepart(dtm_blk);
            end;

            if missing(afd_doff) or missing(afd_doff_time)
                or missing(afd_block_date) or missing(afd_block_time)
                then  delete;
        end;
    run;

    %delete_dataset(_tmp_trans_);
    %delete_dataset(_tmp_marked_cards)

%mend rpt_extract_prod_data_quarterly;
