/*
 * extract production data for analytic reporting
 *
 * Author: Szczuka, William 2015
 *
 */

%macro rpt_extract_production_data(
    frh_table,            /* CCCA, CSCA, etc */
    type,                 /* daily, monthly */
    wherecond = 1,        /* conditions to subsetting input dataset */
    wts_var = 1           /* weight of each observation */
  );


    %local var_list;
    proc sql noprint;
        select var_list into :var_list /*separated by  " "*/
            from devdata.frh_table_info_&type.
            where frh_table = "&frh_table";
    quit;

    %let var_list = &var_list;
    
    %put var_list: &var_list;
    /* connect to analytic datasets */
    %if %nrbquote(&ANALYTIC_EXTRACT_DATASTORE) ne %then 
    %do;
        libname db_lib "&ANALYTIC_EXTRACT_DATASTORE";        
    %end;      
    %else %if %nrbquote(&fake_rh_dir) ne %then %do;
        %fake_connect_to_db(lib = db_lib, dir = &fake_rh_dir);
    %end;
    %else %do;
        %connect_to_db(lib = db_lib);
    %end;

    data &frh_table;
      set DB_LIB.fcm_&frh_table(%fsmetl_getext
           where=(&wherecond )
           keep=&var_list rqo_tran_date);

      %db_specific_data_conversion;

      length wts_var 8;
      wts_var = &wts_var;
    run;


    libname DB_LIB clear;
%mend rpt_extract_production_data;
