/*
 * Macro to parse analytic report configuration file
 * 
 * Author: Song, Qiang <qiang.song@sas.com> 2014
 * 
 */

%macro rptcfg_extract_varlst(xmlfile =, varlst = );
/* DOCSSTRING
This macro extract the list necessary API variables to run analytic reports

#input
- xmlfile: path to the location of the XML configuration file; do not include quotations

#output
- varlst: a SAS dataset containing the list of API variables
    END DOCSTRING */


    %local mapfile;
    %let mapfile = %sysfunc(tranwrd(&xmlfile, .xml, .map));

    libname rptcfg xml "&xmlfile" xmlmap = "&mapfile" access = readonly;

    data &varlst(keep = var_name);

        set
            rptcfg.rpt_variables(keep = var_name var_depending var_recode)
            rptcfg.drl_variables(keep = var_name var_depending var_recode)
            ;            

        if not missing(var_depending) then do;
            i = 1;
            var_name = scan(var_depending, i);
            do while(not missing(var_name));
                output;
                i = i + 1;
                var_name = scan(var_depending, i);
            end;
        end;
        else do;
            output;
        end;
            
    run;

    proc sort nodupkey data = &varlst;
        by _all_;
    run;    
        
    libname rptcfg clear;
%mend;

     
