/****************************************************************************
 * Auxiliary macro which, given any valid SAS variable specification,       *
 * generates a global macro variable containing an enumeration of each      *
 * individual variable. The macro is invoked as:                            *
 *                                                                          *
 * %varlist(sasdata,varnames,__allvars__);                                  *
 *                                                                          *
 * - sasdata represents the SAS data set to be examined;                    *
 * - varnames represents the SAS variable specification (any valid SAS      *
 *   variable specification is accepted).                                   *
 * - __allvars__ represents the name of the global macro variable that will *
 * contain an enumeration of all the variables.                             *
 *                                                                          *
 * Radu Drossu (2003)                                                       *
 ****************************************************************************/

%macro varlist(sasdata,varnames,__allvars__);

%global &__allvars__;

%if &varnames= %then %do;
  data __tmpresults0__;
    set &sasdata(drop=_all_ obs=1) end=eof;
    if eof then call symput("&__allvars__",trim(' '));
  run;
%end;
%else %if &varnames=_all_ %then %do;
  data __tmpresults0__;
    set &sasdata(keep=_all_ obs=1);
  run;

  proc contents data=__tmpresults0__ out=__tmpresults0__(keep=name)
       position noprint;
  run;

  data _null_;
    set __tmpresults0__(keep=name) end=eof;
    length myvars $32767;
    retain myvars "";
    myvars=trim(myvars) || " " || lowcase(trim(name));
    if eof then call symput("&__allvars__", trim(myvars));
  run;
%end;
%else %do;
  data __tmpresults0__;
    set &sasdata(keep=&varnames obs=1);
  run;

  proc contents data=__tmpresults0__ out=__tmpresults0__(keep=name)
       position noprint;
  run;

  data _null_;
    set __tmpresults0__(keep=name) end=eof;
    length myvars $32767;
    retain myvars "";
    myvars=trim(myvars) || " " || lowcase(trim(name));
    if eof then call symput("&__allvars__", trim(myvars));
  run;
%end;

proc datasets;
  delete __tmpresults0__;
run;

%mend varlist;
